# Generated by Django 4.2.3 on 2023-08-02 23:56

from django.db import migrations, models
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0068_remove_historicalhba1cresult_history_user_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.CharField(
                choices=[
                    (
                        "1",
                        "Chief executives, managers, senior officials and legislators",
                    ),
                    ("2", "Professionals, technicians and associate professionals"),
                    ("4", "Clerks"),
                    ("5", "Service workers and shop sale workers"),
                    ("6", "Skilled agricultural and fishery workers"),
                    ("7", "Unskilled agricultural and fishery workers"),
                    ("8", "Craft and related workers"),
                    ("9", "Plant and machine operators and assemblers"),
                    ("10", "Elementary occupations"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=100,
                null=True,
                verbose_name="If OTHER type of employment, specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_employment_type",
            field=models.CharField(
                choices=[
                    (
                        "1",
                        "Chief executives, managers, senior officials and legislators",
                    ),
                    ("2", "Professionals, technicians and associate professionals"),
                    ("4", "Clerks"),
                    ("5", "Service workers and shop sale workers"),
                    ("6", "Skilled agricultural and fishery workers"),
                    ("7", "Unskilled agricultural and fishery workers"),
                    ("8", "Craft and related workers"),
                    ("9", "Plant and machine operators and assemblers"),
                    ("10", "Elementary occupations"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your type of employment?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.CharField(
                choices=[
                    (
                        "1",
                        "Chief executives, managers, senior officials and legislators",
                    ),
                    ("2", "Professionals, technicians and associate professionals"),
                    ("4", "Clerks"),
                    ("5", "Service workers and shop sale workers"),
                    ("6", "Skilled agricultural and fishery workers"),
                    ("7", "Unskilled agricultural and fishery workers"),
                    ("8", "Craft and related workers"),
                    ("9", "Plant and machine operators and assemblers"),
                    ("10", "Elementary occupations"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=100,
                null=True,
                verbose_name="If OTHER type of employment, specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_employment_type",
            field=models.CharField(
                choices=[
                    (
                        "1",
                        "Chief executives, managers, senior officials and legislators",
                    ),
                    ("2", "Professionals, technicians and associate professionals"),
                    ("4", "Clerks"),
                    ("5", "Service workers and shop sale workers"),
                    ("6", "Skilled agricultural and fishery workers"),
                    ("7", "Unskilled agricultural and fishery workers"),
                    ("8", "Craft and related workers"),
                    ("9", "Plant and machine operators and assemblers"),
                    ("10", "Elementary occupations"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your type of employment?",
            ),
        ),
    ]



# Generated by Django 4.1.7 on 2023-05-02 22:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_group", "0010_alter_patientgroup_managers"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpatientfollowupcall",
            name="respondent",
            field=models.CharField(
                choices=[
                    ("patient", "Patient"),
                    ("family", "Family"),
                    ("friend", "friend"),
                    ("OTHER", "other"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="patientfollowupcall",
            name="respondent",
            field=models.CharField(
                choices=[
                    ("patient", "Patient"),
                    ("family", "Family"),
                    ("friend", "friend"),
                    ("OTHER", "other"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
            ),
        ),
    ]



from __future__ import annotations

from typing import Dict

from django.db import IntegrityError
from django.test import TestCase
from edc_constants.constants import FEMALE, MALE, NO, YES
from edc_utils import get_utcnow

from intecomm_consent.models import SubjectConsent
from intecomm_lists.models import ConsentRefusalReasons, ScreeningRefusalReasons
from intecomm_screening.exceptions import AlreadyConsentedError
from intecomm_screening.forms import (
    ConsentRefusalForm,
    PatientLogForm,
    SubjectScreeningForm,
)
from intecomm_screening.models import ConsentRefusal, SubjectScreening
from intecomm_screening.models.subject_screening import SubjectScreeningError
from intecomm_screening.tests.intecomm_test_case_mixin import IntecommTestCaseMixin


class TestConsentRefusalModel(IntecommTestCaseMixin, TestCase):
    def test_consent_refusal_ok(self):
        subject_screening = self.get_subject_screening(
            patient_log_options=dict(willing_to_screen=YES)
        )
        self.assertEqual(subject_screening.reasons_ineligible, None)
        self.assertTrue(subject_screening.eligible)
        consent_refusal = self.get_consent_refusal(
            screening_identifier=subject_screening.screening_identifier
        )
        self.assertEqual(
            consent_refusal.screening_identifier,
            subject_screening.screening_identifier,
        )

    def test_consent_refusal_is_unique_by_screening_identifier(self):
        subject_screening = self.get_subject_screening()
        self.get_consent_refusal(screening_identifier=subject_screening.screening_identifier)
        self.assertRaises(
            IntegrityError,
            self.get_consent_refusal,
            screening_identifier=subject_screening.screening_identifier,
        )

    def test_refusing_consent_after_consenting_raises(self):
        subject_screening = self.get_subject_screening()
        self.get_subject_consent(subject_screening)
        self.assertRaises(
            AlreadyConsentedError,
            self.get_consent_refusal,
            screening_identifier=subject_screening.screening_identifier,
        )


class TestConsentRefusalForm(IntecommTestCaseMixin, TestCase):
    @staticmethod
    def get_refusal_data(screening_identifier: str | None = None) -> Dict:
        refusal_reason = ConsentRefusalReasons.objects.all()[0]
        return {
            "screening_identifier": screening_identifier,
            "report_datetime": get_utcnow(),
            "reason": refusal_reason,
            "other_reason": "",
        }

    def test_consent_refusal_ok(self):
        subject_screening = self.get_subject_screening(
            patient_log_options=dict(willing_to_screen=YES)
        )
        form = ConsentRefusalForm(
            data=self.get_refusal_data(
                screening_identifier=subject_screening.screening_identifier
            ),
            instance=ConsentRefusal(),
        )
        form.is_valid()
        self.assertEqual(form._errors, {})
        form.save()
        self.assertEqual(ConsentRefusal.objects.all().count(), 1)

    def test_screening_form_raises_if_patient_log_willing_to_screen_is_no(self):
        patient_log = self.get_patient_log(
            willing_to_screen=NO,
            screening_refusal_reason=ScreeningRefusalReasons.objects.get(
                name="dont_have_time"
            ),
        )
        form = SubjectScreeningForm(
            initial=dict(patient_log=patient_log), instance=SubjectScreening()
        )
        form.is_valid()
        self.assertIsNone(form._errors)
        self.assertRaises(
            SubjectScreeningError, self.get_subject_screening, patient_log=patient_log
        )

    def test_patient_log_cannot_be_changed_after_screen(self):
        patient_log = self.get_patient_log(willing_to_screen=YES, gender=FEMALE)
        self.get_subject_screening(patient_log=patient_log, gender=FEMALE)
        form = PatientLogForm(data=dict(gender=MALE), instance=patient_log)
        form.is_valid()
        self.assertIn("Patient has already screened", str(form._errors.get("__all__")))

    def test_refusal_after_already_refused_raises(self):
        subject_screening = self.get_subject_screening(
            patient_log_options=dict(willing_to_screen=YES)
        )
        refusal_form = ConsentRefusalForm(
            data=self.get_refusal_data(
                screening_identifier=subject_screening.screening_identifier
            ),
            instance=ConsentRefusal(),
        )
        refusal_form.is_valid()
        self.assertEqual(refusal_form._errors, {})
        refusal_form.save()
        self.assertEqual(ConsentRefusal.objects.all().count(), 1)

        refusal_form_two = ConsentRefusalForm(
            data=self.get_refusal_data(
                screening_identifier=subject_screening.screening_identifier
            ),
            instance=ConsentRefusal(),
        )
        refusal_form_two.is_valid()
        self.assertEqual(
            ["Consent Refusal with this Screening identifier already exists."],
            refusal_form_two._errors.get("screening_identifier"),
        )
        with self.assertRaises(ValueError):
            refusal_form_two.save()

        self.assertEqual(ConsentRefusal.objects.all().count(), 1)

    def test_raises_if_add_consent_refusal_after_already_consented(self):
        subject_screening = self.get_subject_screening(
            patient_log_options=dict(willing_to_screen=YES)
        )
        self.get_subject_consent(subject_screening=subject_screening)
        subject_screening.refresh_from_db()
        self.assertEqual(SubjectConsent.objects.all().count(), 1)

        refusal_form = ConsentRefusalForm(
            data=self.get_refusal_data(
                screening_identifier=subject_screening.screening_identifier
            ),
            instance=ConsentRefusal(),
        )
        refusal_form.is_valid()
        self.assertIn("__all__", refusal_form._errors)
        self.assertIn(
            "Not allowed. Subject has already consented. See subject ",
            refusal_form._errors.get("__all__")[0],
        )
        self.assertIn(
            subject_screening.subject_identifier,
            refusal_form._errors.get("__all__")[0],
        )
        with self.assertRaises(ValueError):
            refusal_form.save()
        self.assertEqual(ConsentRefusal.objects.all().count(), 0)

    def test_refusal_if_not_eligible_raises(self):
        subject_screening = self.get_subject_screening()
        subject_screening.in_care_6m = False
        subject_screening.save()
        self.assertEqual(SubjectScreening.objects.all().count(), 1)

        refusal_form = ConsentRefusalForm(
            data=self.get_refusal_data(
                screening_identifier=subject_screening.screening_identifier
            ),
            instance=ConsentRefusal(),
        )
        refusal_form.is_valid()
        self.assertIn(
            "Not allowed. Subject is not eligible.",
            refusal_form._errors.get("__all__")[0],
        )
        self.assertIn(
            subject_screening.screening_identifier,
            refusal_form._errors.get("__all__")[0],
        )
        with self.assertRaises(ValueError):
            refusal_form.save()
        self.assertEqual(ConsentRefusal.objects.all().count(), 0)



from django import forms
from intecomm_form_validators.subject import DmReviewFormValidator

from ..models import DmReview
from .mixins import CrfModelFormMixin


class DmReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DmReviewFormValidator

    class Meta:
        model = DmReview
        fields = "__all__"



from django.contrib import admin, messages
from edc_utils import get_utcnow

from ..models import PatientLogReportPrintHistory
from ..reports import PatientLogReport, PatientLogReportError


@admin.action(description="Print patient reference")
def render_pdf_action(modeladmin, request, queryset, **kwargs):  # noqa
    report = None
    if queryset.count() > 1:
        messages.add_message(
            request, messages.ERROR, "Select only one patient to print and try again"
        )
    else:
        for obj in queryset:
            try:
                report = PatientLogReport(
                    patient_log=obj, user=request.user
                ).render_to_response()
            except PatientLogReportError as e:
                messages.add_message(request, messages.ERROR, str(e))
                break
            PatientLogReportPrintHistory.objects.create(
                patient_log_identifier=obj.patient_log_identifier,
                printed_datetime=get_utcnow(),
                printed_user=request.user.username,
            )
            obj.printed = True
            obj.save(update_fields=["printed"])
            messages.add_message(
                request,
                messages.SUCCESS,
                f"Successfully printed patient reference for {obj.patient_log_identifier}.",
            )
    return report



from __future__ import annotations

from django.db import models
from edc_model.models import BaseUuidModel
from edc_next_appointment.choices import APPT_DATE_INFO_SOURCES
from edc_next_appointment.model_mixins import NextAppointmentCrfModelMixin

from ..model_mixins import CrfModelMixin


class NextAppointment(NextAppointmentCrfModelMixin, CrfModelMixin, BaseUuidModel):
    # TODO: Remove field after migrations are squashed/reset
    old_health_facility = models.CharField(
        max_length=100,
        null=True,
    )

    # TODO: Remove field after migrations are squashed/reset
    info_source_old = models.CharField(
        verbose_name="What is the source of this appointment date",
        max_length=15,
        choices=APPT_DATE_INFO_SOURCES,
        null=True,
        blank=False,
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Next Appointment"
        verbose_name_plural = "Next Appointments"



# Generated by Django 4.2.3 on 2023-07-11 02:28

from django.db import migrations, models
import edc_model.models.fields.duration
import edc_model.validators.date


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0052_alter_bloodresultsfbc_managers_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalhivreview",
            name="arv_initiated",
        ),
        migrations.RemoveField(
            model_name="historicalhivreview",
            name="arv_initiation_actual_date",
        ),
        migrations.RemoveField(
            model_name="hivreview",
            name="arv_initiated",
        ),
        migrations.RemoveField(
            model_name="hivreview",
            name="arv_initiation_actual_date",
        ),
        migrations.AddField(
            model_name="historicalhivreview",
            name="rx_init",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="Yes",
                max_length=15,
                verbose_name="Has the patient started antiretroviral therapy (ART)?",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivreview",
            name="rx_init_ago",
            field=edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalhivreview",
            name="rx_init_calculated_date",
            field=models.DateField(
                editable=False,
                help_text="Calculated based on response to `rx_init_ago`",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Estimated date started antiretroviral therapy (ART)",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivreview",
            name="rx_init_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date started antiretroviral therapy (ART)",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivreview",
            name="rx_init_date_is_estimated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="N/A",
                editable=False,
                max_length=15,
                verbose_name="Was antiretroviral therapy (ART) start date estimated?",
            ),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="rx_init",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="Yes",
                max_length=15,
                verbose_name="Has the patient started antiretroviral therapy (ART)?",
            ),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="rx_init_ago",
            field=edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="rx_init_calculated_date",
            field=models.DateField(
                editable=False,
                help_text="Calculated based on response to `rx_init_ago`",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Estimated date started antiretroviral therapy (ART)",
            ),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="rx_init_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date started antiretroviral therapy (ART)",
            ),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="rx_init_date_is_estimated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="N/A",
                editable=False,
                max_length=15,
                verbose_name="Was antiretroviral therapy (ART) start date estimated?",
            ),
        ),
    ]



from edc_model_wrapper import ModelWrapper


class ConsentRefusalModelWrapper(ModelWrapper):
    model = "intecomm_screening.consentrefusal"
    next_url_attrs = ["screening_identifier"]
    next_url_name = "screening_listboard_url"

    @property
    def pk(self):
        return str(self.object.pk)

    @property
    def screening_identifier(self):
        return self.object.screening.screening_identifier



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from ..admin_site import intecomm_subject_admin
from ..models import FamilyHistory
from .modeladmin_mixins import CrfModelAdmin


@admin.register(FamilyHistory, site=intecomm_subject_admin)
class FamilyHistoryAdmin(CrfModelAdmin):
    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Part 1",
            {"fields": ("htn_in_household", "dm_in_household", "hiv_in_household")},
        ),
        (
            "Part 2",
            {
                "fields": (
                    "high_bp_bs_tf",
                    "overweight_tf",
                    "salty_foods_tf",
                    "excercise_tf",
                    "take_medicine_tf",
                    "stop_htn_meds_tf",
                    "traditional_htn_tf",
                    "stop_dm_meds_tf",
                    "traditional_dm_tf",
                    "dm_cause_tf",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "htn_in_household": admin.VERTICAL,
        "dm_in_household": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
        "hiv_in_household": admin.VERTICAL,
        "high_bp_bs_tf": admin.VERTICAL,
        "overweight_tf": admin.VERTICAL,
        "salty_foods_tf": admin.VERTICAL,
        "excercise_tf": admin.VERTICAL,
        "take_medicine_tf": admin.VERTICAL,
        "stop_htn_meds_tf": admin.VERTICAL,
        "traditional_htn_tf": admin.VERTICAL,
        "stop_dm_meds_tf": admin.VERTICAL,
        "traditional_dm_tf": admin.VERTICAL,
        "dm_cause_tf": admin.VERTICAL,
    }



from __future__ import annotations

import re
from typing import TYPE_CHECKING

from django.urls import reverse
from django_audit_fields import ModelAdminAuditFieldsMixin
from django_revision.modeladmin_mixin import ModelAdminRevisionMixin
from edc_consent.utils import get_remove_patient_names_from_countries
from edc_constants.constants import UUID_PATTERN
from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.mixins import (
    ModelAdminBypassDefaultFormClsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminInstitutionMixin,
    ModelAdminNextUrlRedirectMixin,
    ModelAdminRedirectAllToChangelistMixin,
    ModelAdminRedirectOnDeleteMixin,
    TemplatesModelAdminMixin,
)
from edc_notification import NotificationModelAdminMixin

from intecomm_sites import all_sites

if TYPE_CHECKING:
    from intecomm_screening.models import PatientLog, SubjectScreening


class BaseModelAdminMixin(
    TemplatesModelAdminMixin,
    ModelAdminRedirectOnDeleteMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminRevisionMixin,
    ModelAdminInstitutionMixin,
    ModelAdminNextUrlRedirectMixin,
    NotificationModelAdminMixin,
    ModelAdminAuditFieldsMixin,
    ModelAdminBypassDefaultFormClsMixin,
    SimpleHistoryAdmin,
):
    show_cancel = True
    view_on_site = False
    save_on_top = True


class RedirectAllToPatientLogModelAdminMixin(ModelAdminRedirectAllToChangelistMixin):
    changelist_url = "intecomm_screening_admin:intecomm_screening_patientlog_changelist"
    change_search_field_name = "screening_identifier"
    add_search_field_name = "screening_identifier"


class ChangeListTopBarModelAdminMixin:
    changelist_top_bar_selected: str = None
    changelist_top_bar_add_url: str = None

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["changelist_top_bar_selected"] = self.changelist_top_bar_selected
        extra_context["changelist_top_bar_add_url"] = reverse(self.changelist_top_bar_add_url)
        return super().changelist_view(request, extra_context=extra_context)


class InitialDataModelAdminMixin:
    def initial_form_data(self, request, obj: PatientLog | SubjectScreening) -> dict:
        """Return a dict of initial data that may or may not
        include legal and familiar name.

        Initial is carried over from the previous model. For example:
          * PatientLog data -> SubjectScreening;
          * SubjectScreening -> SubjectConsent
        """
        dct = dict(
            initials=obj.initials,
            hospital_identifier=obj.hospital_identifier,
            site=obj.site.id,
            gender=obj.gender,
            age_in_years=obj.age_in_years,
        )
        if self.include_names(request):
            legal_name = obj.legal_name
            if re.match(UUID_PATTERN, obj.legal_name):
                legal_name = None
            familiar_name = obj.familiar_name
            if re.match(UUID_PATTERN, obj.familiar_name):
                familiar_name = None
            dct.update(
                legal_name=legal_name,
                familiar_name=familiar_name,
            )
        return dct

    @staticmethod
    def include_names(request) -> bool:
        """Return True if legal and familiar name may be included"""
        include_names = True
        for country in get_remove_patient_names_from_countries():
            if request.site.id in [s.site_id for s in all_sites.get(country)]:
                include_names = False
                break
        return include_names



# from django.contrib import admin
# from django.utils.html import format_html
# from edc_sites import get_site_name
# from edc_sites.admin import SiteModelAdminMixin
#
# from intecomm_sites import all_sites
#
# from ..admin_site import intecomm_screening_admin
# from ..forms import HealthFacilityForm
# from ..models import HealthFacility
# from .modeladmin_mixins import BaseModelAdminMixin, ChangeListTopBarModelAdminMixin
#
#
# @admin.register(HealthFacility, site=intecomm_screening_admin)
# class HealthFacilityAdmin(
#     SiteModelAdminMixin, ChangeListTopBarModelAdminMixin, BaseModelAdminMixin
# ):
#     form = HealthFacilityForm
#     show_object_tools = True
#     change_list_template: str = "intecomm_screening/admin/healthfacility_change_list.html"
#     change_list_title = HealthFacility._meta.verbose_name_plural
#     change_list_note = "These are health facilites within a hospital"
#
#     changelist_top_bar_selected = "healthfacility"
#     changelist_top_bar_add_url = (
#         "intecomm_screening_admin:intecomm_screening_healthfacility_add"
#     )
#
#     fieldsets = (
#         (
#             None,
#             {
#                 "fields": (
#                     "report_datetime",
#                     "name",
#                     "health_facility_type",
#                     "health_facility_type_other",
#                 )
#             },
#         ),
#         (
#             "Clinic days",
#             {"fields": ("mon", "tue", "wed", "thu", "fri", "sat")},
#         ),
#         (
#             "Location",
#             {
#                 "description": "Provide this information if available",
#                 "fields": (
#                     "distance",
#                     "gps",
#                     "latitude",
#                     "longitude",
#                 ),
#             },
#         ),
#         (
#             "Notes",
#             {
#                 "fields": ("notes",),
#             },
#         ),
#     )
#
#     list_display = (
#         "name",
#         "health_facility_type",
#         "clinic_days",
#         "distance",
#         "map",
#     )
#
#     list_filter = (
#         "report_datetime",
#         "health_facility_type",
#     )
#
#     radio_fields = {
#         "health_facility_type": admin.VERTICAL,
#     }
#
#     search_fields = (
#         "name",
#         "health_facility_type__name",
#     )
#
#     @admin.display(description="Map")
#     def map(self, obj=None):
#         if obj.latitude and obj.longitude:
#             return format_html(
#                 f'<A href="https://www.google.com/maps/@{obj.latitude},{obj.longitude},15z">'
#                 '<i class="fas fa-location-dot"></i>'
#             )
#         return None
#
#     @staticmethod
#     def site_name(obj=None):
#         get_site_name(obj.site.id, all_sites)
#         return obj.name
#
#     @admin.display(description="Clinic Days")
#     def clinic_days(self, obj=None) -> str:
#         days = []
#         if obj.mon:
#             days.append("Mon")
#         if obj.tue:
#             days.append("Tue")
#         if obj.wed:
#             days.append("Wed")
#         if obj.thu:
#             days.append("Thu")
#         if obj.fri:
#             days.append("Fri")
#         if obj.sat:
#             days.append("Sat")
#         return ",".join(days) if days else "unknown"



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_action_item.forms import ActionItemFormMixin
from edc_form_validators import FormValidatorMixin
from edc_registration.models import RegisteredSubject
from edc_sites.forms import SiteModelFormMixin

from ..models import SubjectReconsent


class SubjectReconsentForm(
    SiteModelFormMixin, FormValidatorMixin, ActionItemFormMixin, forms.ModelForm
):
    subject_identifier = forms.CharField(
        label="Subject identifier",
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean(self):
        cleaned_data = super().clean()
        try:
            RegisteredSubject.objects.get(
                subject_identifier=cleaned_data.get("subject_identifier"),
                identity=cleaned_data.get("identity"),
            )
        except ObjectDoesNotExist:
            raise forms.ValidationError({"identity": "Identity number does not match."})
        return cleaned_data

    class Meta:
        model = SubjectReconsent
        fields = "__all__"



from edc_lab import ProcessingProfile, disposable, wb

fbc_processing = ProcessingProfile(name="FBC", aliquot_type=wb)

chemistry_processing = ProcessingProfile(name="Chem", aliquot_type=wb)

chemistry_alt_processing = ProcessingProfile(name="Chem + ALT", aliquot_type=wb)

hba1c_processing = ProcessingProfile(name="HbA1c", aliquot_type=wb)

blood_glucose_processing = ProcessingProfile(name="Blood Glucose", aliquot_type=wb)

poc_processing = ProcessingProfile(name="POC", aliquot_type=disposable)



from __future__ import annotations

from typing import Type

import inflect
from django.db import models
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE

from ...choices import DISCLOSURE, IMPACT_SEVERITY, IMPACT_STATUS

inflect = inflect.engine()


def disclosure_model_mixin_factory(
    *prefixes: str,
) -> Type[models.Model]:
    """Returns an abstract model class"""

    class AbstractModel(models.Model):
        class Meta:
            abstract = True

    attrs = {}
    for prefix in prefixes:
        do_str = "Do any of" if inflect.singular_noun(prefix) else "Does"
        attrs.update(
            {
                prefix: models.CharField(
                    verbose_name=(
                        f"{do_str} your {prefix.upper()} know about your participation in the "
                        "study?"
                    ),
                    max_length=25,
                    choices=YES_NO_NA,
                ),
                f"{prefix}_disclosure": models.CharField(
                    verbose_name="If Yes, how did they find out?",
                    max_length=25,
                    choices=DISCLOSURE,
                    default=NOT_APPLICABLE,
                    help_text=(
                        "(e.g., they saw study material, or I had to disclose DM/HTN/HIV"
                    ),
                ),
            }
        )

    for name, fld_cls in attrs.items():
        AbstractModel.add_to_class(name, fld_cls)
    return AbstractModel


def impact_model_mixin_factory(
    *prefixes: str, lead_verbose_name: str | None = None
) -> Type[models.Model]:
    """Returns an abstract model class"""

    class AbstractModel(models.Model):
        class Meta:
            abstract = True

    attrs = {}
    for prefix in prefixes:
        attrs.update(
            {
                f"{prefix}_impact": models.CharField(
                    verbose_name=lead_verbose_name or f"... with your {prefix}?",
                    max_length=25,
                    choices=YES_NO_NA,
                ),
                f"{prefix}_impact_severity": models.CharField(
                    verbose_name="Severity of impact",
                    max_length=25,
                    choices=IMPACT_SEVERITY,
                    default=NOT_APPLICABLE,
                ),
                f"{prefix}_impact_status": models.CharField(
                    verbose_name="Status of impact",
                    max_length=25,
                    choices=IMPACT_STATUS,
                    default=NOT_APPLICABLE,
                ),
                f"{prefix}_impact_help": models.CharField(
                    verbose_name="Would you like help",
                    max_length=25,
                    choices=YES_NO_NA,
                    default=NOT_APPLICABLE,
                ),
                f"{prefix}_impact_referal": models.CharField(
                    verbose_name="Has the participant been referred for further help",
                    max_length=25,
                    choices=YES_NO_NA,
                    default=NOT_APPLICABLE,
                ),
            }
        )

    for name, fld_cls in attrs.items():
        AbstractModel.add_to_class(name, fld_cls)
    return AbstractModel



from django import forms
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import FormValidator
from edc_visit_schedule.utils import raise_if_baseline

from ..models import ClinicalNote


class ClinicalNoteFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_baseline(self.cleaned_data.get("subject_visit"))


class ClinicalNoteForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = ClinicalNoteFormValidator

    class Meta:
        model = ClinicalNote
        fields = "__all__"



from django.db import models
from edc_model.models import BaseUuidModel
from edc_utils import get_utcnow


class PatientLogReportPrintHistory(BaseUuidModel):
    patient_log_identifier = models.CharField(max_length=36)
    printed_datetime = models.DateTimeField(default=get_utcnow)
    printed_user = models.CharField(max_length=25)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Patient Log Report Print History"
        verbose_name_plural = "Patient Log Report Print History"



# Generated by Django 4.1.7 on 2023-05-10 00:52
import sys

from django.db import migrations
from django.db.migrations import RunPython
from django.db.models import Q
from tqdm import tqdm

from intecomm_screening.identifiers import FilingIdentifier, PatientLogIdentifier


def update_identifiers(apps, schema_editor):
    sys.stdout.write(
        "\nUpdating filing_identifier and patient_log_identifier if still UUIDs ...\n"
    )
    qs = apps.get_model("intecomm_screening.patientlog").objects.exclude(
        Q(filing_identifier__startswith="F") | Q(patient_log_identifier__startswith="P")
    )
    sys.stdout.write(f"Found {qs.count()} records to update.\n")
    for obj in tqdm(qs, total=qs.count()):
        obj.filing_identifier = FilingIdentifier(site_id=obj.site_id).identifier
        obj.patient_log_identifier = PatientLogIdentifier(site_id=obj.site_id).identifier
        obj.save_base(update_fields=["filing_identifier", "patient_log_identifier"])
    sys.stdout.write("\nDone.\n")


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0029_alter_historicalpatientlog_filing_identifier_and_more"),
    ]

    operations = [RunPython(update_identifiers)]



from edc_visit_schedule import Schedule

from ..constants import FACILITY_SCHEDULE
from .visits import get_followup_visit, visit00, visit12

facility_schedule = Schedule(
    name=FACILITY_SCHEDULE,
    verbose_name="Facility-based integrated care",
    onschedule_model="intecomm_prn.onscheduleinte",
    offschedule_model="intecomm_prn.offscheduleinte",
    consent_model="intecomm_consent.subjectconsent",
    appointment_model="edc_appointment.appointment",
)

visits = [visit00]
for month in range(1, 12):
    visits.append(get_followup_visit(month))
visits.append(visit12)

for visit in visits:
    facility_schedule.add_visit(visit=visit)



from django.contrib import admin
from edc_adverse_event.modeladmin_mixins import AeFollowupModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_ae_admin
from ..forms import AeFollowupForm
from ..models import AeFollowup


@admin.register(AeFollowup, site=intecomm_ae_admin)
class AeFollowupAdmin(SiteModelAdminMixin, AeFollowupModelAdminMixin, SimpleHistoryAdmin):
    form = AeFollowupForm



from edc_constants.constants import (
    ESTIMATED,
    MEASURED,
    NEVER,
    NO,
    NOT_APPLICABLE,
    OTHER,
    PATIENT,
    YES,
)
from edc_dx_review.constants import DIET_LIFESTYLE, DRUGS, THIS_CLINIC
from edc_facility.constants import HEALTH_FACILITY
from edc_reportable import (
    MILLIGRAMS_PER_DECILITER,
    MILLIMOLES_PER_LITER,
    MILLIMOLES_PER_LITER_DISPLAY,
)
from edc_visit_tracking.constants import MISSED_VISIT, SCHEDULED, UNSCHEDULED

from .constants import COMMUNITY_CLINIC, GTE_3HRS, NURSE, SITTING

ALCOHOL_CONSUMPTION = (
    ("occasionally", "Occasionally"),
    ("1_2_per_week", "1-2 times a week"),
    ("3_4_per_week", "3-4 times a week"),
    ("daily", "Daily"),
    (NOT_APPLICABLE, "Not applicable"),
)

CARE_ACCESS = (
    (THIS_CLINIC, "Patient comes to this facility for their care"),
    (OTHER, "Patient goes to a different clinic"),
    (NOT_APPLICABLE, "Not applicable"),
)

CARE_DELIVERY = (
    (HEALTH_FACILITY, "Health facility"),
    (COMMUNITY_CLINIC, "Community clinic"),
    (OTHER, "Other facility, please specify below ..."),
    (NOT_APPLICABLE, "Not applicable"),
)

DISCLOSURE = (
    ("voluntary", "Voluntary disclosure"),
    ("involuntary", "Involuntary disclosure"),
    ("by_others", "Others told them without permission"),
    (NOT_APPLICABLE, "Not applicable"),
)

EDUCATION = (
    ("no_formal_education", "No Formal Education"),
    ("primary", "Up to primary"),
    ("secondary", "Up to secondary / high school"),
    ("post_secondary", "College or similar higher institution"),
    ("tertiary", "university educated"),
)

EMPLOYMENT_STATUS = (
    ("professional", "Professional / office work / business"),
    ("manual_work", "Skilled / Unskilled manual work"),
    ("housewife", "Housewife"),
    ("unemployed", "Not working / seeking work"),
    ("unemployed_not_seeking", "Not working / NOT seeking work"),
    ("retired", "Retired"),
    (OTHER, "Other, please specify"),
)

HOUSEHOLD_YES_NO_CHOICES = (
    (NO, "No"),
    ("yes_spouse", "Yes, my spouse"),
    ("yes_parents", "Yes, one of my parents living with me"),
    ("yes_relative", "Yes, another relative living with me"),
)

HTN_MANAGEMENT = (
    (DRUGS, "Drugs / Medicine"),
    (DIET_LIFESTYLE, "Diet and lifestyle alone"),
)

IMPACT_SEVERITY = (
    ("minor", "Minor"),
    ("moderate", "Moderate"),
    ("major", "Major"),
    (NOT_APPLICABLE, "Not applicable"),
)

IMPACT_STATUS = (
    ("resolved", "Resolved"),
    ("ongoing", "Ongoing"),
    (NOT_APPLICABLE, "Not applicable"),
)

MISSED_PILLS = (
    ("today", "today"),
    ("yesterday", "yesterday"),
    ("earlier_this_week", "earlier this week"),
    ("last_week", "last week"),
    ("lt_month_ago", "less than a month ago"),
    ("gt_month_ago", "more than a month ago"),
    (NEVER, "have never missed taking my study pills"),
)

MARITAL_STATUS = (
    ("married", "Married or living with someone"),
    ("single", "Single"),
    ("divorced", "Divorced"),
    ("widowed", "Widow / Spinster"),
)


# *********************************
ACTIVITY_CHOICES = (
    ("working", "Working"),
    ("studying", "Studying"),
    ("caring_for_children", "Caring for children"),
    ("house_maintenance", "House maintenance"),
    ("nothing", "Nothing"),
    (OTHER, "Other, please specify"),
)

CARD_TYPE_CHOICES = (
    ("paper_based", "Paper-based"),
    ("electronic", "Electronic"),
    ("both", "Both"),
    (NOT_APPLICABLE, "Not Applicable"),
)

CHILDCARE_CHOICES = (
    (NOT_APPLICABLE, "Not applicable"),
    ("working", "Working"),
    ("studying", "Studying"),
    ("caring_for_children", "Caring for children"),
    ("house_maintenance", "House maintenance"),
    ("nothing", "Nothing"),
    (OTHER, "Other, specify"),
)

GLUCOSE_UNITS = (
    (MILLIGRAMS_PER_DECILITER, MILLIGRAMS_PER_DECILITER),
    (MILLIMOLES_PER_LITER, MILLIMOLES_PER_LITER_DISPLAY),
    (NOT_APPLICABLE, "Not applicable"),
)

HCF_PRESCRIPTION_COLLECTION_CHOICES = (
    (YES, "Yes"),
    (NO, "No, I buy my own drugs"),
    (NOT_APPLICABLE, "Not applicable"),
)

INFO_SOURCE = (
    ("patient", "Patient"),
    ("patient_and_outpatient", "Patient, hospital notes and/or outpatient card"),
    ("patient_representive", "Patient Representative (e.family member, friend)"),
    ("hospital_notes", "Hospital notes"),
    ("outpatient_cards", "Outpatient cards"),
    ("collateral_history", "Collateral History from relative/guardian"),
    (NOT_APPLICABLE, "Not applicable"),
    (OTHER, "Other"),
)

MISSED_VISIT_CALLER_CHOICES = (
    (NURSE, "Nurse"),
    (OTHER, "Other"),
    (NOT_APPLICABLE, "Not applicable"),
)

PHYSICAL_ACTIVITY = (
    ("retired", "Retired"),
    (SITTING, "Mostly sitting"),
    ("standing_or_walking", "Mostly standing or walking"),
    ("physical_effort", "Definite physical effort"),
    ("vigorous_physical_activity", "Vigorous physical activity"),
)

PHYSICAL_ACTIVITY_HOURS = (
    ("none", "None"),
    ("lt_1hr", "Some but less than one hour"),
    ("1-3hr", "1 hour but less than 3 hours"),
    (GTE_3HRS, "3 hours or more"),
)


TRANSPORT_CHOICES = (
    ("bus", "Bus"),
    ("train", "Train"),
    ("ambulance", "Ambulance"),
    ("private_taxi", "Private taxi"),
    ("own_bicycle", "Own bicycle"),
    ("hired_motorbike", "Hired motorbike"),
    ("own_car", "Own car"),
    ("own_motorbike", "Own motorbike"),
    ("hired_bicycle", "Hired bicycle"),
    ("foot", "Foot"),
    (OTHER, "Other, specify"),
)

VISIT_UNSCHEDULED_REASON = (
    ("routine_non_study", "Routine appointment (non-study)"),
    ("patient_unwell_outpatient", "Patient unwell"),
    ("drug_refill", "Drug refill only"),
    (OTHER, "Other"),
    (NOT_APPLICABLE, "Not applicable"),
)

VISIT_REASON = (
    (SCHEDULED, "Scheduled visit (study)"),
    (UNSCHEDULED, "Routine / Unscheduled visit (non-study)"),
    (MISSED_VISIT, "Missed visit"),
)

ESTIMATED_MEASURED_CHOICES = (
    (MEASURED, "Measured"),
    (ESTIMATED, "Estimated"),
    (NOT_APPLICABLE, "Not recorded / Not applicable"),
)

APPT_DATE_INFO_SOURCES = (
    ("health_records", "Health record"),
    (PATIENT, "Patient"),
    ("estimated", "I estimated the date"),
)



from django.apps import apps as django_apps
from django.contrib.auth.models import User
from django.urls import NoReverseMatch, reverse
from django_webtest import WebTest

from intecomm_subject.models import DrugSupplyDm, DrugSupplyHiv, DrugSupplyHtn
from intecomm_subject.models import HealthEconomics as OldHealthEconomics


class TestAdmin(WebTest):
    def setUp(self) -> None:
        super().setUp()
        self.user = User.objects.create_superuser("user_login", "u@example.com", "pass")
        self.user.is_active = True
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.user.refresh_from_db()
        self.exclude_models = [
            DrugSupplyDm,
            DrugSupplyHtn,
            DrugSupplyHiv,
            OldHealthEconomics,
        ]

    def login(self):
        form = self.app.get(reverse("admin:index")).maybe_follow().form
        form["username"] = self.user.username
        form["password"] = "pass"  # nosec B105
        return form.submit()

    def test_admin_changelists_ok(self):
        self.login()
        app_config = django_apps.get_app_config("intecomm_subject")
        models = [
            model for model in app_config.get_models() if model not in self.exclude_models
        ]
        urls = []
        errors = []
        for model in models:
            app_name, model_name = model._meta.label_lower.split(".")
            if "historical" not in model_name:
                url_name = f"{app_name}_admin:{app_name}_{model_name}_changelist"
                try:
                    url = reverse(url_name)
                except NoReverseMatch:
                    errors.append(url_name)
                else:
                    urls.append((url, model))
        if errors:
            as_str = "\n".join(errors)
            self.fail(f"NoReverseMatch unexpectedly raised. Got urls \n{as_str}.")
        for url, model in urls:
            response = self.app.get(url, user=self.user, status=200)
            self.assertTrue(
                model._meta.verbose_name_plural in response.text,
                msg=f"{model._meta.verbose_name_plural} not found in changelist html",
            )



from django.db import models
from edc_model.models import BaseUuidModel
from edc_rx.model_mixins import DrugSupplyModelMixin

from intecomm_lists.models import HtnTreatments

from .drug_refill_htn import DrugRefillHtn


class DrugSupplyHtn(DrugSupplyModelMixin, BaseUuidModel):
    drug_refill = models.ForeignKey(DrugRefillHtn, on_delete=models.PROTECT)

    drug = models.ForeignKey(HtnTreatments, on_delete=models.PROTECT)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Drug Supply: Hypertension"
        verbose_name_plural = "Drug Supply: Hypertension"



# Generated by Django 4.1.7 on 2023-04-04 00:58

from django.db import migrations
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0021_htninitialreview_managed_by_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalhtninitialreview",
            name="managed_by_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="htninitialreview",
            name="managed_by_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
    ]



from typing import Tuple

from django.apps import apps as django_apps
from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from edc_model_admin.mixins import ModelAdminLimitToSelectedForeignkey
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_screening_admin
from ..forms import PatientCallForm
from ..models import PatientCall, PatientLog
from .list_filters import LastApptListFilter, NextApptListFilter
from .modeladmin_mixins import (
    BaseModelAdminMixin,
    ChangeListTopBarModelAdminMixin,
    RedirectAllToPatientLogModelAdminMixin,
)


@admin.register(PatientCall, site=intecomm_screening_admin)
class PatientCallAdmin(
    SiteModelAdminMixin,
    ModelAdminLimitToSelectedForeignkey,
    RedirectAllToPatientLogModelAdminMixin,
    ChangeListTopBarModelAdminMixin,
    BaseModelAdminMixin,
):
    form = PatientCallForm
    show_object_tools = True
    list_per_page = 5

    # TemplatesModelAdminMixin attr
    change_list_template: str = "intecomm_screening/admin/patientcall_change_list.html"
    change_list_title = PatientCall._meta.verbose_name

    # ChangeListTopBarModelAdminMixin attrs
    changelist_top_bar_selected = "patientcall"
    changelist_top_bar_add_url = "intecomm_screening_admin:intecomm_screening_patientcall_add"

    # RedirectAllToPatientLogModelAdminMixin attrs
    change_search_field_name = "patient_log__patient_log_identifier"
    add_search_field_name = "patient_log__patient_log_identifier"

    # SiteModelAdminMixin attr
    limit_related_to_current_site = ["patient_log"]

    # ModelAdminLimitToSelectedForeignkey attrs
    limit_fk_field_to_selected = [("patient_log", PatientLog)]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "patient_log",
                    "report_datetime",
                )
            },
        ),
        (
            "Contact numbers",
            {
                "fields": (
                    "contact_number",
                    "alt_contact_number",
                )
            },
        ),
        (
            "Patient Status",
            {
                "fields": (
                    "answered",
                    "respondent",
                    "survival_status",
                    "catchment_area",
                )
            },
        ),
        (
            "Patient Care",
            {
                "fields": (
                    "last_appt_date",
                    "last_attend_clinic",
                    # "last_attend_clinic_other",
                    "next_appt_date",
                )
            },
        ),
        (
            "Notes and followup",
            {
                "fields": (
                    "call_again",
                    "comment",
                )
            },
        ),
    )

    list_display = (
        "report_datetime",
        "patient_log_link",
        "answered",
        "respondent",
        "alive",
        "in_area",
        "call_again",
        "last_appt",
        "next_appt",
    )

    list_filter = (
        "report_datetime",
        LastApptListFilter,
        NextApptListFilter,
        "answered",
        "call_again",
        "respondent",
        "survival_status",
        "catchment_area",
    )

    radio_fields = {
        "answered": admin.VERTICAL,
        "respondent": admin.VERTICAL,
        "survival_status": admin.VERTICAL,
        "catchment_area": admin.VERTICAL,
        "call_again": admin.VERTICAL,
    }

    search_fields = (
        "patient_log__id",
        "patient_log__legal_name__exact",
        "patient_log__familiar_name__exact",
    )

    @admin.display(description="Last Appt", ordering="last_appt_date")
    def last_appt(self, obj=None):
        return obj.last_appt_date

    @admin.display(description="Next Appt", ordering="next_appt_date")
    def next_appt(self, obj=None):
        return obj.next_appt_date

    @admin.display(description="Patient", ordering="patient_log")
    def patient_log_link(self, obj=None):
        url = reverse("intecomm_screening_admin:intecomm_screening_patientlog_changelist")
        url = f"{url}?q={obj.patient_log.id}"
        return format_html(f'<A href="{url}">{obj.patient_log}</a>')

    @property
    def patient_log_model_cls(self):
        return django_apps.get_model("intecomm_screening.patientlog")

    @admin.display(description="Alive", ordering="survival_status")
    def alive(self, obj=None):
        return obj.survival_status

    @admin.display(description="In Area", ordering="catchment_area")
    def in_area(self, obj=None):
        return obj.catchment_area

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        readonly = super().get_readonly_fields(request, obj=obj)
        if obj and "patient_log" not in readonly:
            readonly = readonly + ("patient_log",)
        return readonly



import os
import sys
from importlib.metadata import version
from pathlib import Path

import environ
from edc_appointment.constants import SCHEDULED_APPT, UNSCHEDULED_APPT
from edc_constants.constants import COMPLETE
from edc_protocol_incident.constants import PROTOCOL_INCIDENT
from edc_utils import get_datetime_from_env

env = environ.Env(
    AWS_ENABLED=(bool, False),
    CDN_ENABLED=(bool, False),
    CELERY_ENABLED=(bool, False),
    DATABASE_SQLITE_ENABLED=(bool, False),
    DJANGO_AUTO_CREATE_KEYS=(bool, False),
    DJANGO_CRYPTO_FIELDS_TEMP_PATH=(bool, False),
    DJANGO_CSRF_COOKIE_SECURE=(bool, True),
    DJANGO_DEBUG=(bool, False),
    DJANGO_EDC_BOOTSTRAP=(int, 3),
    DJANGO_EMAIL_ENABLED=(bool, False),
    DJANGO_EMAIL_USE_TLS=(bool, True),
    DJANGO_LIVE_SYSTEM=(bool, False),
    DJANGO_LOGGING_ENABLED=(bool, True),
    DJANGO_SESSION_COOKIE_SECURE=(bool, True),
    DJANGO_USE_I18N=(bool, False),
    DJANGO_USE_TZ=(bool, True),
    DEFENDER_ENABLED=(bool, False),
    # EDC_RANDOMIZATION_REGISTER_DEFAULT_RANDOMIZER=(bool, True),
    EDC_LABEL_BROWSER_PRINT_PAGE_AUTO_BACK=(bool, True),
    TWILIO_ENABLED=(bool, False),
)

DEBUG = env("DJANGO_DEBUG")

if LOGGING_ENABLED := env("DJANGO_LOGGING_ENABLED"):
    from .logging import *  # noqa

BASE_DIR = str(Path(os.path.dirname(os.path.abspath(__file__))).parent.parent)
ENV_DIR = str(Path(os.path.dirname(os.path.abspath(__file__))).parent.parent)

# copy your .env file from .envs/ to BASE_DIR
if "test" in sys.argv:
    env.read_env(os.path.join(ENV_DIR, ".env-tests"))
    print(f"Reading env from {os.path.join(BASE_DIR, '.env-tests')}")
else:
    if not os.path.exists(os.path.join(ENV_DIR, ".env")):
        raise FileExistsError(
            f"Environment file does not exist. Got `{os.path.join(ENV_DIR, '.env')}`"
        )
    env.read_env(os.path.join(ENV_DIR, ".env"))

DEBUG = env("DJANGO_DEBUG")

SECRET_KEY = env.str("DJANGO_SECRET_KEY")

APP_NAME = env.str("DJANGO_APP_NAME")

LIVE_SYSTEM = env.str("DJANGO_LIVE_SYSTEM")

ETC_DIR = env.str("DJANGO_ETC_FOLDER")

TEST_DIR = os.path.join(BASE_DIR, APP_NAME, "tests")

# INTERNAL_IPS = ["127.0.0.1"] # for djdt
ALLOWED_HOSTS = ["*"]  # env.list('DJANGO_ALLOWED_HOSTS')

ENFORCE_RELATED_ACTION_ITEM_EXISTS = False

DEFAULT_APPOINTMENT_TYPE = "hospital"

LOGIN_URL = "/accounts/login/"
LOGOUT_REDIRECT_URL = "/accounts/login/"

DEFENDER_ENABLED = env("DEFENDER_ENABLED")

INSTALLED_APPS = [
    "intecomm_edc.apps.AdminConfig",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sites",
    "defender",
    "multisite",
    "fontawesomefree",
    "django_crypto_fields.apps.AppConfig",
    "django_revision.apps.AppConfig",
    # "django_extensions",
    # "debug_toolbar",
    "logentry_admin",
    "simple_history",
    "storages",
    "edc_action_item.apps.AppConfig",
    "edc_appointment.apps.AppConfig",
    "edc_auth.apps.AppConfig",
    "edc_adverse_event.apps.AppConfig",
    "edc_consent.apps.AppConfig",
    "edc_crf.apps.AppConfig",
    "edc_reportable.apps.AppConfig",
    "edc_lab.apps.AppConfig",
    "edc_visit_schedule.apps.AppConfig",
    "edc_visit_tracking.apps.AppConfig",
    "edc_dx.apps.AppConfig",
    "edc_dx_review.apps.AppConfig",
    "edc_device.apps.AppConfig",
    "edc_dashboard.apps.AppConfig",
    "edc_data_manager.apps.AppConfig",
    "edc_export.apps.AppConfig",
    "edc_facility.apps.AppConfig",
    "edc_fieldsets.apps.AppConfig",
    "edc_form_validators.apps.AppConfig",
    "edc_he.apps.AppConfig",
    "edc_lab_dashboard.apps.AppConfig",
    "edc_label.apps.AppConfig",
    "edc_list_data.apps.AppConfig",
    "edc_listboard.apps.AppConfig",
    "edc_identifier.apps.AppConfig",
    "edc_locator.apps.AppConfig",
    "edc_metadata.apps.AppConfig",
    "edc_model.apps.AppConfig",
    "edc_model_fields.apps.AppConfig",
    "edc_model_admin.apps.AppConfig",
    "edc_navbar.apps.AppConfig",
    "edc_next_appointment.apps.AppConfig",
    "edc_notification.apps.AppConfig",
    "edc_offstudy.apps.AppConfig",
    "edc_pharmacy.apps.AppConfig",
    "edc_pdutils.apps.AppConfig",
    "edc_protocol.apps.AppConfig",
    "edc_protocol_incident.apps.AppConfig",
    "edc_prn.apps.AppConfig",
    "edc_qol.apps.AppConfig",
    "edc_randomization.apps.AppConfig",
    "edc_reference.apps.AppConfig",
    "edc_refusal.apps.AppConfig",
    "edc_registration.apps.AppConfig",
    "edc_pdf_reports.apps.AppConfig",
    "edc_review_dashboard.apps.AppConfig",
    "edc_rx.apps.AppConfig",
    "edc_screening.apps.AppConfig",
    "edc_sites.apps.AppConfig",
    "edc_subject_dashboard.apps.AppConfig",
    "edc_timepoint.apps.AppConfig",
    "edc_unblinding.apps.AppConfig",
    "edc_form_describer.apps.AppConfig",
    "edc_adherence.apps.AppConfig",
    "canned_views.apps.AppConfig",
    "intecomm_rando.apps.AppConfig",
    "intecomm_consent.apps.AppConfig",
    "intecomm_lists.apps.AppConfig",
    "intecomm_dashboard.apps.AppConfig",
    "intecomm_facility.apps.AppConfig",
    "intecomm_labs.apps.AppConfig",
    "intecomm_subject.apps.AppConfig",
    "intecomm_visit_schedule.apps.AppConfig",
    "intecomm_ae.apps.AppConfig",
    "intecomm_auth.apps.AppConfig",
    "intecomm_prn.apps.AppConfig",
    "intecomm_export.apps.AppConfig",
    "intecomm_group.apps.AppConfig",
    "intecomm_screening.apps.AppConfig",
    "intecomm_sites.apps.AppConfig",
    "intecomm_edc.apps.AppConfig",
]

if not DEFENDER_ENABLED:
    INSTALLED_APPS.pop(INSTALLED_APPS.index("defender"))

MIDDLEWARE = [
    # "debug_toolbar.middleware.DebugToolbarMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "multisite.middleware.DynamicSiteMiddleware",
    "django.contrib.sites.middleware.CurrentSiteMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "defender.middleware.FailedLoginMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

if not DEFENDER_ENABLED:
    MIDDLEWARE.pop(MIDDLEWARE.index("defender.middleware.FailedLoginMiddleware"))

MIDDLEWARE.extend(
    [
        "edc_protocol.middleware.ProtocolMiddleware",
        "edc_dashboard.middleware.DashboardMiddleware",
        "edc_subject_dashboard.middleware.DashboardMiddleware",
        "edc_lab_dashboard.middleware.DashboardMiddleware",
        "edc_adverse_event.middleware.DashboardMiddleware",
        "edc_listboard.middleware.DashboardMiddleware",
        "edc_review_dashboard.middleware.DashboardMiddleware",
        # 'simple_history.middleware.HistoryRequestMiddleware'
    ]
)

ROOT_URLCONF = f"{APP_NAME}.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "edc_model_admin.context_processors.admin_theme",
                "edc_constants.context_processor.constants",
                "edc_appointment.context_processors.constants",
                "edc_visit_tracking.context_processors.constants",
            ]
        },
    }
]

if env("DATABASE_SQLITE_ENABLED"):
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
        }
    }

else:
    DATABASES = {"default": env.db()}
# be secure and clear DATABASE_URL since it is no longer needed.
DATABASE_URL = None

if env.str("DJANGO_CACHE") == "redis":
    CACHES = {
        "default": {
            "BACKEND": "django_redis.cache.RedisCache",
            "LOCATION": "redis://127.0.0.1:6379/1",
            # "LOCATION": "unix://[:{DJANGO_REDIS_PASSWORD}]@/path/to/socket.sock?db=0",
            "OPTIONS": {
                "CLIENT_CLASS": "django_redis.client.DefaultClient",
                "PASSWORD": env.str("DJANGO_REDIS_PASSWORD"),
            },
            "KEY_PREFIX": f"{APP_NAME}",
        }
    }
    SESSION_ENGINE = "django.contrib.sessions.backends.cache"
    SESSION_CACHE_ALIAS = "default"
    DJANGO_REDIS_IGNORE_EXCEPTIONS = True
    DJANGO_REDIS_LOG_IGNORED_EXCEPTIONS = True
elif env.str("DJANGO_CACHE") == "memcached":
    CACHES = {
        "default": {
            "BACKEND": "django.core.cache.backends.memcached.PyLibMCCache",
            "LOCATION": "unix:/tmp/memcached.sock",
        }
    }
    SESSION_ENGINE = "django.contrib.sessions.backends.cached_db"

WSGI_APPLICATION = f"{APP_NAME}.wsgi.application"

# Password validation
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTHENTICATION_BACKENDS = ["edc_auth.backends.ModelBackendWithSite"]

PASSWORD_HASHERS = [
    "django.contrib.auth.hashers.Argon2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher",
    "django.contrib.auth.hashers.BCryptSHA256PasswordHasher",
]

AUTH_PASSWORD_VALIDATORS = [
    {"NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"},
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
        "OPTIONS": {"min_length": 20},
    },
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]
# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

USE_I18N = False  # disable trans
USE_L10N = False  # set to False so DATE formats below are used
USE_TZ = True
LANGUAGE_CODE = env.str("DJANGO_LANGUAGE_CODE")  # ignored if USE_L10N = False
LANGUAGES = [x.split(":") for x in env.list("DJANGO_LANGUAGES")] or (("en", "English"),)
TIME_ZONE = env.str("DJANGO_TIME_ZONE")
DATE_INPUT_FORMATS = ["%Y-%m-%d", "%d/%m/%Y"]
DATETIME_INPUT_FORMATS = [
    "%Y-%m-%d %H:%M:%S",  # '2006-10-25 14:30:59'
    "%Y-%m-%d %H:%M:%S.%f",  # '2006-10-25 14:30:59.000200'
    "%Y-%m-%d %H:%M",  # '2006-10-25 14:30'
    "%Y-%m-%d",  # '2006-10-25'
    "%d/%m/%Y %H:%M:%S",  # '25/10/2006 14:30:59'
    "%d/%m/%Y %H:%M:%S.%f",  # '25/10/2006 14:30:59.000200'
    "%d/%m/%Y %H:%M",  # '25/10/2006 14:30'
    "%d/%m/%Y",  # '25/10/2006'
]
DATE_FORMAT = "j N Y"
DATETIME_FORMAT = "j N Y H:i"
SHORT_DATE_FORMAT = "d/m/Y"
SHORT_DATETIME_FORMAT = "d/m/Y H:i"

REPORT_DATETIME_FIELD_NAME = "report_datetime"

# See also any inte_* or edc_* apps.py
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# edc-appointment
EDC_APPOINTMENT_APPT_TYPE_CHOICES = (
    ("clinic", "IN CLINIC / FACILITY"),
    ("community", "IN COMMUNITY"),
    ("hospital", "In hospital"),
    ("home", "At home"),
    ("telephone", "By telephone"),
)
EDC_APPOINTMENT_APPT_TYPE_DEFAULT = None

EDC_APPOINTMENT_APPT_REASON_CHOICES = (
    (SCHEDULED_APPT, "Scheduled visit (study)"),
    (UNSCHEDULED_APPT, "Routine / Unscheduled (non-study)"),
)

# edc-export
EDC_EXPORT_EXPORT_PII_USERS = env.list("EDC_EXPORT_EXPORT_PII_USERS")
EDC_FACILITY_HEALTH_FACILITY_MODEL = "intecomm_facility.healthfacility"

# edc-he
EDC_HE_ASSETS_MODEL = "intecomm_subject.healtheconomicsassets"
EDC_HE_HOUSEHOLDHEAD_MODEL = "intecomm_subject.healtheconomicshouseholdhead"
EDC_HE_INCOME_MODEL = "intecomm_subject.healtheconomicsincome"
EDC_HE_PATIENT_MODEL = "intecomm_subject.healtheconomicspatient"
EDC_HE_PROPERTY_MODEL = "intecomm_subject.healtheconomicsproperty"

# edc-pdutils
EXPORT_FILENAME_TIMESTAMP_FORMAT = "%Y%m%d"

# django_revision
REVISION = version(APP_NAME)

# enforce https if DEBUG=False!
# Note: will cause "CSRF verification failed. Request aborted"
#       if DEBUG=False and https not configured.
if not DEBUG:
    # CSFR cookies
    CSRF_COOKIE_SECURE = env.str("DJANGO_CSRF_COOKIE_SECURE")
    SECURE_PROXY_SSL_HEADER = env.tuple("DJANGO_SECURE_PROXY_SSL_HEADER")
    SESSION_COOKIE_SECURE = env.str("DJANGO_SESSION_COOKIE_SECURE")
    # other security defaults
    SECURE_SSL_REDIRECT = True
    SECURE_HSTS_SECONDS = 31_536_000
    SECURE_HSTS_INCLUDE_SUBDOMAINS = True
    SECURE_CONTENT_TYPE_NOSNIFF = True
    SECURE_BROWSER_XSS_FILTER = True

# edc_lab and label
LABEL_TEMPLATE_FOLDER = env.str("DJANGO_LABEL_TEMPLATE_FOLDER") or os.path.join(
    BASE_DIR, "label_templates", "2.25x1.25in"
)
CUPS_SERVERS = env.dict("DJANGO_CUPS_SERVERS")

LIST_MODEL_APP_LABEL = env.str("EDC_LIST_MODEL_APP_LABEL")
SUBJECT_APP_LABEL = env.str("EDC_SUBJECT_APP_LABEL")
SUBJECT_SCREENING_MODEL = env.str("EDC_SUBJECT_SCREENING_MODEL")
SUBJECT_CONSENT_MODEL = env.str("EDC_SUBJECT_CONSENT_MODEL")
SUBJECT_LOCATOR_MODEL = "intecomm_prn.subjectlocator"
SUBJECT_REFUSAL_MODEL = env.str("EDC_REFUSAL_SUBJECT_REFUSAL_MODEL")
SUBJECT_REQUISITION_MODEL = env.str("EDC_SUBJECT_REQUISITION_MODEL")
SUBJECT_VISIT_MODEL = env.str("EDC_SUBJECT_VISIT_MODEL")
SUBJECT_VISIT_MISSED_MODEL = env.str("EDC_SUBJECT_VISIT_MISSED_MODEL")
SUBJECT_VISIT_MISSED_REASONS_MODEL = env.str("EDC_SUBJECT_VISIT_MISSED_REASONS_MODEL")
EDC_BLOOD_RESULTS_MODEL_APP_LABEL = "intecomm_subject"

EDC_NAVBAR_DEFAULT = env("EDC_NAVBAR_DEFAULT")

# dashboards
EDC_BOOTSTRAP = env("DJANGO_EDC_BOOTSTRAP")
DASHBOARD_URL_NAMES = env.dict("DJANGO_DASHBOARD_URL_NAMES")
DASHBOARD_BASE_TEMPLATES = env.dict("DJANGO_DASHBOARD_BASE_TEMPLATES")
LAB_DASHBOARD_BASE_TEMPLATES = env.dict("DJANGO_LAB_DASHBOARD_BASE_TEMPLATES")
LAB_DASHBOARD_URL_NAMES = env.dict("DJANGO_LAB_DASHBOARD_URL_NAMES")

# edc-consent
EDC_CONSENT_REMOVE_PATIENT_NAMES_FROM_COUNTRIES = env.list(
    "EDC_CONSENT_REMOVE_PATIENT_NAMES_FROM_COUNTRIES", default=[]
)

# edc-diagnosis
EDC_DX_LABELS = dict(hiv="HIV", dm="Diabetes", htn="Hypertension")

# edc-dx-review
EDC_DX_REVIEW_APP_LABEL = "intecomm_subject"
EDC_DX_REVIEW_LIST_MODEL_APP_LABEL = "edc_dx_review"
# edc_facility
HOLIDAY_FILE = env.str("DJANGO_HOLIDAY_FILE")

# edc-label
EDC_LABEL_BROWSER_PRINT_PAGE_AUTO_BACK = env("EDC_LABEL_BROWSER_PRINT_PAGE_AUTO_BACK")

# edc_model_admin
EDC_MODEL_ADMIN_CSS_THEME = "edc_indigo"

EDC_OFFSTUDY_OFFSTUDY_MODEL = "intecomm_prn.endofstudy"

# edc-protocol-incident
EDC_PROTOCOL_VIOLATION_TYPE = PROTOCOL_INCIDENT

# edc_randomization

EDC_RANDOMIZATION_UNBLINDED_USERS = env.list("EDC_RANDOMIZATION_UNBLINDED_USERS")
EDC_RANDOMIZATION_REGISTER_DEFAULT_RANDOMIZER = False
EDC_RANDOMIZATION_SKIP_VERIFY_CHECKS = True
EDC_RANDOMIZATION_LIST_PATH = env.str("EDC_RANDOMIZATION_LIST_PATH")

# edc-sites
EDC_SITES_MODULE_NAME = env.str("EDC_SITES_MODULE_NAME")

# django-multisite
CACHE_MULTISITE_KEY_PREFIX = APP_NAME
SILENCED_SYSTEM_CHECKS = ["sites.E101"]

# django-defender
# see if env.str("DJANGO_CACHE") == "redis" above
# and that redis server is running
DEFENDER_REDIS_NAME = "default"
DEFENDER_LOCK_OUT_BY_IP_AND_USERNAME = True
DEFENDER_LOCKOUT_TEMPLATE = "edc_auth/bootstrap3/login.html"
DEFENDER_LOGIN_FAILURE_LIMIT = 5

# edc_crf
CRF_STATUS_DEFAULT = COMPLETE

EMAIL_ENABLED = env("DJANGO_EMAIL_ENABLED")
EMAIL_CONTACTS = env.dict("DJANGO_EMAIL_CONTACTS")
if EMAIL_ENABLED:
    EMAIL_HOST = env.str("DJANGO_EMAIL_HOST")
    EMAIL_PORT = env.int("DJANGO_EMAIL_PORT")
    EMAIL_HOST_USER = env.str("DJANGO_EMAIL_HOST_USER")
    EMAIL_HOST_PASSWORD = env.str("DJANGO_EMAIL_HOST_PASSWORD")
    EMAIL_USE_TLS = env("DJANGO_EMAIL_USE_TLS")
    MAILGUN_API_KEY = env("MAILGUN_API_KEY")
    MAILGUN_API_URL = env("MAILGUN_API_URL")

# django_revision
GIT_DIR = BASE_DIR

# django_crypto_fields
KEY_PATH = env.str("DJANGO_KEY_FOLDER")
AUTO_CREATE_KEYS = env("DJANGO_AUTO_CREATE_KEYS")

EXPORT_FOLDER = env.str("DJANGO_EXPORT_FOLDER") or os.path.expanduser("~/")

# django_simple_history
SIMPLE_HISTORY_ENFORCE_HISTORY_MODEL_PERMISSIONS = True

FQDN = env.str("DJANGO_FQDN")  # ???
INDEX_PAGE = env.str("DJANGO_INDEX_PAGE")
INDEX_PAGE_LABEL = env.str("DJANGO_INDEX_PAGE_LABEL")

# edc_adverse_event
ADVERSE_EVENT_ADMIN_SITE = env.str("EDC_ADVERSE_EVENT_ADMIN_SITE")
ADVERSE_EVENT_APP_LABEL = env.str("EDC_ADVERSE_EVENT_APP_LABEL")

# edc_data_manager
DATA_DICTIONARY_APP_LABELS = [
    "intecomm_consent",
    "intecomm_subject",
    "intecomm_prn",
    "intecomm_screening",
    "intecomm_ae",
    "edc_appointment",
    "edc_locator",
    "edc_offstudy",
]

# edc_protocol
EDC_PROTOCOL = env.str("EDC_PROTOCOL")
EDC_PROTOCOL_INSTITUTION_NAME = env.str("EDC_PROTOCOL_INSTITUTION_NAME")
EDC_PROTOCOL_NUMBER = env.str("EDC_PROTOCOL_NUMBER")
# EDC_PROTOCOL_PROJECT_NAME = env.str("EDC_PROTOCOL_PROJECT_NAME")
EDC_PROTOCOL_PROJECT_NAME = "INTECOMM"
EDC_PROTOCOL_STUDY_OPEN_DATETIME = get_datetime_from_env(
    *env.list("EDC_PROTOCOL_STUDY_OPEN_DATETIME")
)
EDC_PROTOCOL_STUDY_CLOSE_DATETIME = get_datetime_from_env(
    *env.list("EDC_PROTOCOL_STUDY_CLOSE_DATETIME")
)
EDC_PROTOCOL_TITLE = env.str("EDC_PROTOCOL_TITLE")

# static / AWS
if env("AWS_ENABLED"):
    # see
    # https://www.digitalocean.com/community/tutorials/
    # how-to-set-up-a-scalable-django-app-with-digitalocean-
    # managed-databases-and-spaces
    AWS_ACCESS_KEY_ID = env.str("AWS_ACCESS_KEY_ID")
    AWS_DEFAULT_ACL = "public-read"
    AWS_SECRET_ACCESS_KEY = env.str("AWS_SECRET_ACCESS_KEY")
    AWS_STORAGE_BUCKET_NAME = env.str("AWS_STORAGE_BUCKET_NAME")
    AWS_S3_CUSTOM_DOMAIN = env.str("AWS_S3_CUSTOM_DOMAIN")
    AWS_S3_ENDPOINT_URL = env.str("AWS_S3_ENDPOINT_URL")
    AWS_S3_OBJECT_PARAMETERS = {"CacheControl": "max-age=86400"}
    AWS_LOCATION = env.str("AWS_LOCATION")
    AWS_IS_GZIPPED = True
    STATICFILES_STORAGE = "storages.backends.s3boto3.S3Boto3Storage"
    STATIC_URL = f"{os.path.join(AWS_S3_CUSTOM_DOMAIN, AWS_LOCATION)}/"
    STATIC_ROOT = ""
elif DEBUG:
    STATIC_URL = "static/"  # env.str("DJANGO_STATIC_URL")
    STATIC_ROOT = env.str("DJANGO_STATIC_ROOT") or os.path.expanduser(
        "~/source/edc_source/intecomm-edc/static/"
    )
else:
    # run collectstatic, check nginx LOCATION
    STATIC_URL = env.str("DJANGO_STATIC_URL")
    STATIC_ROOT = env.str("DJANGO_STATIC_ROOT")

# CELERY

if "test" in sys.argv:

    class DisableMigrations:
        def __contains__(self, item):
            return True

        def __getitem__(self, item):
            return None

    MIGRATION_MODULES = DisableMigrations()
    PASSWORD_HASHERS = ("django.contrib.auth.hashers.MD5PasswordHasher",)
    DEFAULT_FILE_STORAGE = "inmemorystorage.InMemoryStorage"


INTECOMM_MIN_GROUP_SIZE = 14
INTECOMM_MIN_GROUP_SIZE_FOR_RATIO = 9
INTECOMM_MAX_MONTHS_TO_NEXT_APPT = 7



from .ae import AeListboardView, DeathReportListboardView
from .screening import ListboardView as ScreenGroupListboardView
from .subject import CommSubjectListboardView
from .subject import DashboardView as SubjectDashboardView
from .subject import InteSubjectListboardView



from .end_of_study_admin import EndOfStudyAdmin
from .loss_to_followup_admin import LossToFollowupAdmin
from .offschedule_comm_admin import OffScheduleCommAdmin
from .offschedule_inte_admin import OffScheduleInteAdmin
from .onschedule_comm_admin import OnScheduleCommAdmin
from .onschedule_inte_admin import OnScheduleInteAdmin
from .protocol_incident_admin import ProtocolIncidentAdmin
from .subject_locator_admin import SubjectLocatorAdmin
from .subject_transfer_admin import SubjectTransferAdmin



# Generated by Django 4.1.2 on 2022-12-01 03:01

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0014_alter_historicalpatientlog_familiar_name_and_more"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="healthtalklog",
            name="unique_health_facility_report_date",
        ),
        migrations.AlterUniqueTogether(
            name="healthtalklog",
            unique_together={("health_facility", "report_date")},
        ),
    ]



from edc_crf.model_mixins import CrfStatusModelMixin
from edc_lab.model_mixins import RequisitionModelMixin
from edc_model.models import BaseUuidModel
from edc_reference.model_mixins import ReferenceModelMixin


class SubjectRequisition(
    RequisitionModelMixin, CrfStatusModelMixin, ReferenceModelMixin, BaseUuidModel
):
    class Meta(RequisitionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Subject requisition"
        verbose_name_plural = "Subject requisitions"



from django import forms
from edc_rx.modelform_mixins import DrugSupplyNcdModelFormMixin

from intecomm_lists.models import DmTreatments

from ..models import DrugSupplyDm


class DrugSupplyDmForm(DrugSupplyNcdModelFormMixin, forms.ModelForm):
    list_model_cls = DmTreatments
    relation_label = "drugsupplydm"

    class Meta:
        model = DrugSupplyDm
        fields = "__all__"



from django.contrib.auth.models import User
from django.urls import reverse
from django_webtest import WebTest
from edc_dashboard import url_names

from intecomm_dashboard.views import (
    AeListboardView,
    CommSubjectListboardView,
    DeathReportListboardView,
    InteSubjectListboardView,
    ScreenGroupListboardView,
)
from intecomm_subject.models import DrugSupplyDm, DrugSupplyHiv, DrugSupplyHtn
from intecomm_subject.models import HealthEconomics as OldHealthEconomics


class TestViews(WebTest):
    def setUp(self) -> None:
        super().setUp()
        self.user = User.objects.create_superuser("user_login", "u@example.com", "pass")
        self.user.is_active = True
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.user.refresh_from_db()
        self.exclude_models = [
            DrugSupplyDm,
            DrugSupplyHtn,
            DrugSupplyHiv,
            OldHealthEconomics,
        ]

    def login(self):
        form = self.app.get(reverse("admin:index")).maybe_follow().form
        form["username"] = self.user.username
        form["password"] = "pass"  # nosec B105
        return form.submit()

    def test_listboards_ok(self):
        self.login()

        url_name = url_names.get(ScreenGroupListboardView.listboard_url)
        url = reverse(url_name)
        response = self.app.get(url, user=self.user, status=200)
        self.assertIn(ScreenGroupListboardView.listboard_panel_title, response.text)

        url_name = url_names.get(AeListboardView.listboard_url)
        url = reverse(url_name)
        response = self.app.get(url, user=self.user, status=200)
        self.assertIn(AeListboardView.listboard_panel_title, response.text)

        url_name = url_names.get(DeathReportListboardView.listboard_url)
        url = reverse(url_name)
        response = self.app.get(url, user=self.user, status=200)
        self.assertIn(DeathReportListboardView.listboard_panel_title, response.text)

        url_name = url_names.get(CommSubjectListboardView.listboard_url)
        url = reverse(url_name)
        response = self.app.get(url, user=self.user, status=200)
        self.assertIn(CommSubjectListboardView.listboard_panel_title, response.text)

        url_name = url_names.get(InteSubjectListboardView.listboard_url)
        url = reverse(url_name)
        response = self.app.get(url, user=self.user, status=200)
        self.assertIn(InteSubjectListboardView.listboard_panel_title, response.text)



# Generated by Django 4.1.7 on 2023-03-31 02:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0016_historicalpatientlog_group_identifier_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="group_identifier",
            field=models.CharField(
                blank=True,
                help_text="Auto populated when group is randomized",
                max_length=25,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="group_identifier",
            field=models.CharField(
                blank=True,
                help_text="Auto populated when group is randomized",
                max_length=25,
                null=True,
            ),
        ),
    ]



from intecomm_lists.models import ArvRegimens as Base


class ArvRegimens(Base):
    class Meta:
        proxy = True
        verbose_name = "ARV Regimens"
        verbose_name_plural = "ARV Regimens"



from edc_protocol import Protocol

from .patterns import screening_identifier
from .views import (
    AeListboardView,
    CommSubjectListboardView,
    DeathReportListboardView,
    InteSubjectListboardView,
    ScreenGroupListboardView,
    SubjectDashboardView,
)

app_name = "intecomm_dashboard"

urlpatterns = InteSubjectListboardView.urls(
    namespace=app_name,
    label="inte_subject_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)
urlpatterns += CommSubjectListboardView.urls(
    namespace=app_name,
    label="comm_subject_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)
urlpatterns += ScreenGroupListboardView.urls(
    namespace=app_name,
    label="screen_group_listboard",
    identifier_label="screening_identifier",
    identifier_pattern=screening_identifier,
)
urlpatterns += SubjectDashboardView.urls(
    namespace=app_name,
    label="subject_dashboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)

urlpatterns += AeListboardView.urls(
    namespace=app_name,
    label="ae_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)
urlpatterns += DeathReportListboardView.urls(
    namespace=app_name,
    label="death_report_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)



# Generated by Django 4.2.3 on 2023-08-03 15:14

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_he", "0004_historicalemployment_historicaleducation_employment_and_more"),
        ("intecomm_subject", "0080_auto_20230803_1726"),
    ]

    operations = [
        migrations.RenameField(
            model_name="healtheconomicshouseholdhead",
            old_name="hoh_education",
            new_name="hoh_education_old",
        ),
        migrations.RenameField(
            model_name="healtheconomicshouseholdhead",
            old_name="hoh_employment_type",
            new_name="hoh_employment_type_old",
        ),
        migrations.RenameField(
            model_name="healtheconomicspatient",
            old_name="pat_employment_type",
            new_name="pat_employment_type_old",
        ),
        migrations.RenameField(
            model_name="healtheconomicspatient",
            old_name="pat_education",
            new_name="pat_education_old",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicshouseholdhead",
            old_name="hoh_education",
            new_name="hoh_education_old",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicshouseholdhead",
            old_name="hoh_employment_type",
            new_name="hoh_employment_type_old",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicspatient",
            old_name="pat_employment_type",
            new_name="pat_employment_type_old",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicspatient",
            old_name="pat_education",
            new_name="pat_education_old",
        ),
    ]



from django import forms
from django.utils.html import format_html
from edc_consent.utils import get_consent_model_cls
from edc_crf.modelform_mixins import CrfSingletonModelFormMixin
from edc_he.constants import CHF, NHIF
from edc_he.form_validators import HealthEconomicsHouseholdHeadFormValidator
from edc_sites import get_sites_by_country

from intecomm_sites import all_sites

from ...models import HealthEconomicsHouseholdHead
from ..mixins import CrfModelFormMixin


class HealthEconomicsHouseholdHeadForm(
    CrfSingletonModelFormMixin, CrfModelFormMixin, forms.ModelForm
):
    form_validator_cls = HealthEconomicsHouseholdHeadFormValidator

    def clean_hoh_insurance(self):
        hoh_insurance = self.cleaned_data.get("hoh_insurance")
        for obj in hoh_insurance.all():
            uganda_sites = get_sites_by_country(country="uganda", all_sites=all_sites)
            if obj.name in [NHIF, CHF] and self.related_visit.site_id in [
                s.site_id for s in uganda_sites
            ]:
                raise forms.ValidationError(
                    f"Invalid select for your country (Uganda). Got `{obj.display_name}`."
                )

        return hoh_insurance

    @property
    def subject_consent(self):
        return get_consent_model_cls().objects.get(subject_identifier=self.subject_identifier)

    class Meta:
        model = HealthEconomicsHouseholdHead
        fields = "__all__"
        help_texts = {
            "hoh_employment_type": format_html(
                '<div class="form-row"><OL><LI><b>Chief executives, managers, senior '
                "officials and legislators</b> </li>"
                "<LI><b>Professionals, technicians and associate professionals</b>  (e.g. "
                "science/engineering professionals, architects, nurses, doctors, teachers, "
                "technicians, construction/mining supervisors, etc.)</li>"
                "<LI><b>Clerks</b> (e.g. clerical support workers, receptionist, secretary, "
                "postman/woman etc.) </li>"
                "<LI><b>Service workers and shop sale workers</b> (e.g. shop sales, cooks, "
                "waiter/bartenders, hairdressers, caretakers, street food/stall salespersons, "
                "childcare workers, teachers aides, healthcare/personal care "
                "assistants etc.) </li>"
                "<LI><b>Large-scale agricultural, forestry and fishery workers</b> </li>"
                "<LI><b>Subsistence farmers, fishers, etc.</b></li>"
                "<LI><b>Craft and related workers</b> (e.g. builders, plumbers, painters, "
                "mechanics, craftsmen, potters, welders, etc.) </li>"
                "<LI><b>Plant and machine operators and assemblers, drivers</b> (e.g. "
                "factory/plant operators, miners, truck/bus drivers, taxi drivers, "
                "train drivers, etc.) </li>"
                "<LI><b>Elementary occupations</b> (e.g. cleaners, farm pickers/labourers, "
                "rickshaw drivers, builder assistants, hawkers, shoe shiners, street car "
                "cleaners, garbage collectors, street sweepers, etc.) </li></ol></div>"
            )
        }



from __future__ import annotations

from typing import Tuple

from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django_audit_fields.admin import audit_fieldset_tuple
from edc_consent.actions import (
    flag_as_verified_against_paper,
    unflag_as_verified_against_paper,
)
from edc_consent.modeladmin_mixins import (
    ConsentModelAdminMixin,
    PiiNamesModelAdminMixin,
)
from edc_constants.choices import GENDER
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.mixins import ModelAdminProtectPiiMixin
from edc_sites.admin import SiteModelAdminMixin

from intecomm_screening.admin.modeladmin_mixins import (
    InitialDataModelAdminMixin,
    RedirectAllToPatientLogModelAdminMixin,
)
from intecomm_screening.models import SubjectScreening
from intecomm_sites import all_sites

from ..admin_site import intecomm_consent_admin
from ..forms import SubjectConsentForm
from ..models import SubjectConsent


@admin.register(SubjectConsent, site=intecomm_consent_admin)
class SubjectConsentAdmin(
    PiiNamesModelAdminMixin,
    SiteModelAdminMixin,
    InitialDataModelAdminMixin,
    RedirectAllToPatientLogModelAdminMixin,
    ModelAdminProtectPiiMixin,
    ConsentModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = SubjectConsentForm

    name_fields: list[str] = ["legal_name", "familiar_name"]
    name_display_field: str = "familiar_name"
    all_sites = all_sites
    list_per_page = 5

    show_object_tools = False
    show_cancel = True
    change_list_template: str = "intecomm_consent/admin/subjectconsent_change_list.html"

    actions = [
        flag_as_verified_against_paper,
        unflag_as_verified_against_paper,
    ]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "screening_identifier",
                    "subject_identifier",
                    "legal_name",
                    "familiar_name",
                    "initials",
                    "gender",
                    "language",
                    "is_literate",
                    "witness_name",
                    "consent_datetime",
                    "dob",
                    "is_dob_estimated",
                    "identity",
                    "identity_type",
                    "confirm_identity",
                    "is_incarcerated",
                )
            },
        ),
        (
            "Review Questions",
            {
                "fields": (
                    "consent_reviewed",
                    "study_questions",
                    "assessment_score",
                    "consent_signature",
                    "consent_copy",
                ),
                "description": "The following questions are directed to the interviewer.",
            },
        ),
        (
            "Group",
            {"classes": ("collapse",), "fields": ("group_identifier",)},
        ),
        audit_fieldset_tuple,
    )

    search_fields = (
        "subject_identifier",
        "screening_identifier",
        "identity__exact",
        "initials__exact",
        "legal_name__exact",
        "familiar_name__exact",
    )

    radio_fields = {
        "gender": admin.VERTICAL,
        "assessment_score": admin.VERTICAL,
        "consent_copy": admin.VERTICAL,
        "consent_reviewed": admin.VERTICAL,
        "consent_signature": admin.VERTICAL,
        "is_dob_estimated": admin.VERTICAL,
        "identity_type": admin.VERTICAL,
        "is_incarcerated": admin.VERTICAL,
        "is_literate": admin.VERTICAL,
        "language": admin.VERTICAL,
        "study_questions": admin.VERTICAL,
    }

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == "gender":
            kwargs["choices"] = GENDER
        return super().formfield_for_choice_field(db_field, request, **kwargs)

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        readonly_fields = super().get_readonly_fields(request, obj=obj)
        if "group_identifier" not in readonly_fields:
            readonly_fields += ("group_identifier",)
        return readonly_fields

    def get_changeform_initial_data(self, request) -> dict:
        dct = super().get_changeform_initial_data(request)
        try:
            subject_screening = SubjectScreening.objects.get(
                screening_identifier=dct.get("screening_identifier")
            )
        except ObjectDoesNotExist:
            pass
        else:
            dct.update(**self.initial_form_data(request, subject_screening))
        return dct



# Generated by Django 4.2.3 on 2023-08-03 15:28

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0083_auto_20230803_1828"),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_education_old",
            field=models.CharField(
                max_length=25,
                verbose_name="Highest level of education completed by the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type_old",
            field=models.CharField(
                max_length=25, verbose_name="Household head’s type of employment"
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity_old",
            field=models.CharField(
                max_length=25, verbose_name="What is the household head’s ethnic background?"
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_education_old",
            field=models.CharField(
                max_length=25, verbose_name="Highest level of education completed?"
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_employment_type_old",
            field=models.CharField(
                max_length=25, verbose_name="What is your type of employment?"
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_education_old",
            field=models.CharField(
                max_length=25,
                verbose_name="Highest level of education completed by the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type_old",
            field=models.CharField(
                max_length=25, verbose_name="Household head’s type of employment"
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity_old",
            field=models.CharField(
                max_length=25, verbose_name="What is the household head’s ethnic background?"
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_education_old",
            field=models.CharField(
                max_length=25, verbose_name="Highest level of education completed?"
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_employment_type_old",
            field=models.CharField(
                max_length=25, verbose_name="What is your type of employment?"
            ),
        ),
    ]



from edc_auth.site_auths import site_auths
from edc_screening.auth_objects import SCREENING, SCREENING_SUPER, SCREENING_VIEW

from .auth_objects import screening_codenames

site_auths.update_group(*screening_codenames, name=SCREENING, no_delete=True)
site_auths.update_group(*screening_codenames, name=SCREENING_SUPER)
site_auths.update_group(*screening_codenames, name=SCREENING_VIEW, view_only=True)



# Generated by Django 4.2.3 on 2023-07-26 18:20

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_facility", "0002_auto_20230726_2030"),
        ("intecomm_screening", "0053_rename_idenfifierformat_identifierformat_and_more"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="healthtalklog",
            unique_together={("health_facility", "report_date")},
        ),
    ]



from __future__ import annotations

import random
from copy import deepcopy
from datetime import datetime
from uuid import uuid4
from zoneinfo import ZoneInfo

from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from edc_appointment.constants import IN_PROGRESS_APPT, INCOMPLETE_APPT
from edc_appointment.models import Appointment
from edc_appointment.tests.test_case_mixins import AppointmentTestCaseMixin
from edc_constants.constants import (
    COMPLETE,
    DM,
    FEMALE,
    HIV,
    HTN,
    MALE,
    NO,
    NOT_APPLICABLE,
    YES,
)
from edc_facility.import_holidays import import_holidays
from edc_list_data.site_list_data import site_list_data
from edc_metadata.models import CrfMetadata
from edc_randomization.site_randomizers import site_randomizers
from edc_sites import add_or_update_django_sites, get_sites_by_country
from edc_utils import get_utcnow
from edc_visit_schedule.constants import DAY1
from edc_visit_tracking.constants import SCHEDULED
from faker import Faker
from model_bakery import baker
from model_bakery.baker import make_recipe
from tqdm import tqdm

from intecomm_lists.models import Conditions
from intecomm_screening.models import (
    ConsentRefusal,
    PatientGroup,
    PatientGroupRando,
    PatientLog,
    SubjectScreening,
)
from intecomm_sites.sites import fqdn
from intecomm_sites.tests.site_test_case_mixin import SiteTestCaseMixin
from intecomm_subject.models import ClinicalReview, ClinicalReviewBaseline, SubjectVisit

fake = Faker()
now = datetime(2019, 5, 1).astimezone(ZoneInfo("UTC"))
tomorrow = now + relativedelta(days=1)


def get_eligible_options(patient_log: PatientLog):
    hiv_dx = True if patient_log.conditions.filter(name=HIV) else False
    dm_dx = True if patient_log.conditions.filter(name=DM) else False
    htn_dx = True if patient_log.conditions.filter(name=HTN) else False
    return dict(
        report_datetime=now,
        patient_log=patient_log,
        legal_name=patient_log.legal_name,
        familiar_name=patient_log.familiar_name,
        initials=patient_log.initials,
        hospital_identifier=patient_log.hospital_identifier,
        gender=patient_log.gender,
        age_in_years=patient_log.age_in_years,
        in_care_6m=YES,
        lives_nearby=YES,
        staying_nearby_6=YES,
        pregnant=NOT_APPLICABLE,
        excluded_by_bp_history=NO,
        excluded_by_gluc_history=NO,
        requires_acute_care=NO,
        hiv_dx=YES if hiv_dx else NO,
        hiv_dx_6m=YES if hiv_dx else NOT_APPLICABLE,
        hiv_dx_ago="1y" if hiv_dx else None,
        art_unchanged_3m=YES if hiv_dx else NOT_APPLICABLE,
        art_stable=YES if hiv_dx else NOT_APPLICABLE,
        art_adherent=YES if hiv_dx else NOT_APPLICABLE,
        dm_dx=YES if dm_dx else NO,
        dm_dx_6m=YES if dm_dx else NO,
        dm_dx_ago="1y" if dm_dx else None,
        dm_complications=NO if dm_dx else NOT_APPLICABLE,
        htn_dx=YES if htn_dx else NO,
        htn_dx_6m=YES if htn_dx else NO,
        htn_dx_ago="1y" if htn_dx else None,
        htn_complications=NO if htn_dx else NOT_APPLICABLE,
        sys_blood_pressure_one=120,
        dia_blood_pressure_one=80,
        sys_blood_pressure_two=120,
        dia_blood_pressure_two=80,
        consent_ability=YES,
        unsuitable_for_study=NO,
        unsuitable_agreed=NOT_APPLICABLE,
    )


class IntecommTestCaseMixin(AppointmentTestCaseMixin, SiteTestCaseMixin):
    fqdn = fqdn

    default_sites = get_sites_by_country("tanzania")

    site_names = [s.name for s in default_sites]

    import_randomization_list = True

    sid_count_for_tests = 5

    @classmethod
    def setUpTestData(cls):
        import_holidays(test=True)
        add_or_update_django_sites(sites=get_sites_by_country("tanzania"))
        if cls.import_randomization_list:
            randomizer_cls = site_randomizers.get("default")
            randomizer_cls.import_list(
                verbose=False, sid_count_for_tests=cls.sid_count_for_tests
            )
        site_list_data.initialize()
        site_list_data.autodiscover()

    @classmethod
    def get_patient_log(
        cls,
        legal_name: str | None = None,
        familiar_name: str | None = None,
        initials: str | None = None,
        gender: str | None = None,
        age_in_years: int | None = None,
        hospital_identifier: str | None = None,
        conditions: list[Conditions] | None = None,
        **kwargs,
    ):
        opts = dict(
            legal_name=legal_name or "NAMEA AAA",
            familiar_name=familiar_name or "NAMEA",
            initials=initials or "NA",
            gender=gender or FEMALE,
            age_in_years=age_in_years or 20,
            hospital_identifier=hospital_identifier or uuid4().hex,
            contact_number="1234567890",
        )
        opts.update(**kwargs)
        patient_log = make_recipe("intecomm_screening.patientlog", **opts)
        conditions = conditions or [HIV]
        for condition in conditions:
            patient_log.conditions.add(Conditions.objects.get(name=condition))
        return patient_log

    @classmethod
    def get_subject_screening(
        cls,
        patient_log: PatientLog | None = None,
        report_datetime: datetime | None = None,
        eligibility_datetime: datetime | None = None,
        gender: str | None = None,
        age_in_years: int | None = None,
        conditions: list[Conditions] | None = None,
        patient_log_options: dict | None = None,
    ):
        patient_log_opt = dict(
            gender=gender,
            age_in_years=age_in_years,
            conditions=conditions,
        )
        patient_log_opt.update(**(patient_log_options or {}))
        patient_log = patient_log or cls.get_patient_log(**patient_log_opt)
        eligible_options = deepcopy(get_eligible_options(patient_log=patient_log))
        eligible_options.update(report_datetime=report_datetime or now)
        subject_screening = SubjectScreening.objects.create(
            patient_log_identifier=patient_log.patient_log_identifier,
            user_created="erikvw",
            user_modified="erikvw",
            **eligible_options,
        )
        screening_identifier = subject_screening.screening_identifier

        subject_screening = SubjectScreening.objects.get(
            screening_identifier=screening_identifier
        )
        patient_log.screening_identifier = screening_identifier
        patient_log.save()

        if eligibility_datetime:
            subject_screening.eligibility_datetime = eligibility_datetime
            subject_screening.save()
            subject_screening = SubjectScreening.objects.get(
                screening_identifier=screening_identifier
            )
        return subject_screening

    @staticmethod
    def get_subject_consent(subject_screening, consent_datetime=None, site_id=None):
        return baker.make_recipe(
            "intecomm_consent.subjectconsent",
            user_created="erikvw",
            user_modified="erikvw",
            screening_identifier=subject_screening.screening_identifier,
            legal_name=subject_screening.legal_name,
            familiar_name=subject_screening.familiar_name,
            initials=subject_screening.initials,
            gender=subject_screening.gender,
            dob=(now.date() - relativedelta(years=subject_screening.age_in_years)),
            identity=subject_screening.hospital_identifier,
            confirm_identity=subject_screening.hospital_identifier,
            site=Site.objects.get(id=site_id or settings.SITE_ID),
            consent_datetime=consent_datetime or subject_screening.report_datetime,
            version=1,
        )

    @staticmethod
    def get_crf_metadata(
        subject_visit, model: str | None = None, entry_status: str | None = None
    ):
        opts = dict(
            subject_identifier=subject_visit.subject_identifier,
            visit_code=subject_visit.visit_code,
            visit_code_sequence=subject_visit.visit_code_sequence,
        )
        if model:
            opts.update(model=model)
        if entry_status:
            opts.update(entry_status=entry_status)
        return CrfMetadata.objects.filter(**opts)

    @staticmethod
    def get_consent_refusal(screening_identifier: str):
        return ConsentRefusal.objects.create(
            user_created="jw",
            user_modified="jw",
            screening_identifier=screening_identifier,
        )

    @classmethod
    def get_patient_group(
        cls, report_datetime: datetime | None = None, conditions: list[str] | None = None
    ):
        conditions = conditions or ([HIV] * 4) + ([HTN] * 5) + ([DM] * 5)
        for i, condition_name in tqdm(enumerate(conditions), total=len(conditions)):
            first_name = fake.first_name()
            last_name = fake.last_name()
            legal_name = f"{first_name} {last_name}"
            initials = f"{first_name[0]}{last_name[0]}"
            report_datetime = report_datetime or get_utcnow()
            gender = random.choice([FEMALE, MALE])  # nosec B311
            age_in_years = random.choice(  # nosec B311
                [20, 25, 30, 35, 40, 45, 50, 55, 60, 65]  # nosec B311
            )
            patient_log_options = dict(
                report_datetime=report_datetime,
                legal_name=legal_name,
                familiar_name=legal_name,
                initials=initials,
                gender=gender,
                age_in_years=age_in_years,
                hospital_identifier=uuid4().hex,
                contact_number=f"12345678{i}",
                conditions=[condition_name],
            )
            subject_screening = cls.get_subject_screening(
                patient_log_options=patient_log_options, report_datetime=report_datetime
            )
            cls.get_subject_consent(subject_screening, consent_datetime=report_datetime)

        patient_group = PatientGroup.objects.create(
            name="BRANDX", report_datetime=report_datetime
        )
        total = PatientLog.objects.filter(conditions__name__in=[HIV]).count()
        for obj in tqdm(PatientLog.objects.filter(conditions__name__in=[HIV]), total=total):
            patient_group.hiv_patients.add(obj)
        total = PatientLog.objects.filter(conditions__name__in=[HTN]).count()
        for obj in tqdm(PatientLog.objects.filter(conditions__name__in=[HTN]), total=total):
            patient_group.htn_patients.add(obj)
        total = PatientLog.objects.filter(conditions__name__in=[DM]).count()
        for obj in tqdm(PatientLog.objects.filter(conditions__name__in=[DM]), total=total):
            patient_group.dm_patients.add(obj)
        return patient_group

    @classmethod
    def get_randomized_patient_group(
        cls,
        report_datetime: datetime | None = None,
        patient_group: PatientGroup | None = None,
        conditions: list[str] | None = None,
    ):
        patient_group = patient_group or cls.get_patient_group(
            report_datetime=report_datetime, conditions=conditions
        )
        patient_group.status = COMPLETE
        patient_group.save()

        patient_group = PatientGroupRando.objects.get(id=patient_group.id)
        patient_group.randomize_now = YES
        patient_group.confirm_randomize_now = "RANDOMIZE"
        patient_group.save()
        patient_group.refresh_from_db()
        return patient_group

    @classmethod
    def get_subject_visit(
        cls,
        visit_code: str | None = None,
        subject_identifier: str | None = None,
        report_datetime: datetime | None = None,
    ) -> SubjectVisit:
        visit_code = visit_code or DAY1
        if subject_identifier:
            appointment = Appointment.objects.get(
                visit_code=visit_code, subject_identifier=subject_identifier
            )
        else:
            appointment = Appointment.objects.filter(visit_code=visit_code)[0]
        subject_visit = SubjectVisit.objects.create(
            appointment=appointment,
            report_datetime=report_datetime or appointment.appt_datetime,
            reason=SCHEDULED,
        )
        return subject_visit

    @classmethod
    def get_next_subject_visit(
        cls,
        subject_visit,
        report_datetime: datetime | None = None,
    ):
        appointment = subject_visit.appointment
        appointment.appt_status = INCOMPLETE_APPT
        appointment.save()
        appointment.refresh_from_db()
        next_appointment = appointment.next_by_timepoint
        next_appointment.appt_status = IN_PROGRESS_APPT
        next_appointment.save()
        next_subject_visit = SubjectVisit(
            appointment=next_appointment,
            reason=SCHEDULED,
            report_datetime=report_datetime or next_appointment.appt_datetime,
            visit_code=next_appointment.visit_code,
            visit_code_sequence=next_appointment.visit_code_sequence,
        )
        next_subject_visit.save()
        next_subject_visit.refresh_from_db()
        return next_subject_visit

    @staticmethod
    def get_or_create_clinical_review_baseline(
        patient_log: PatientLog, subject_visit: SubjectVisit
    ) -> ClinicalReviewBaseline:
        cond = [obj.name for obj in patient_log.conditions.all()][0]
        options = {f"{cond.lower()}_dx": YES, f"{cond.lower()}_dx_at_screening": YES}
        try:
            obj = ClinicalReviewBaseline.objects.get(subject_visit=subject_visit)
        except ObjectDoesNotExist:
            for cond in [c for c in [HIV, DM, HTN] if c != cond]:
                options.update(
                    {
                        f"{cond.lower()}_dx": NO,
                        f"{cond.lower()}_dx_at_screening": NOT_APPLICABLE,
                    }
                )
            obj = baker.make_recipe(
                "intecomm_subject.clinicalreviewbaseline",
                subject_visit=subject_visit,
                report_datetime=subject_visit.report_datetime,
                **options,
            )
        return obj

    @staticmethod
    def get_or_create_clinical_review(
        patient_log: PatientLog, subject_visit: SubjectVisit
    ) -> ClinicalReview:
        cond = [obj.name for obj in patient_log.conditions.all()][0]
        options = {f"{cond.lower()}_dx": YES}
        try:
            obj = ClinicalReview.objects.get(subject_visit=subject_visit)
        except ObjectDoesNotExist:
            for cond in [c for c in [HIV, DM, HTN] if c != cond]:
                options.update({f"{cond.lower()}_dx": NO})
            obj = baker.make_recipe(
                "intecomm_subject.clinicalreview",
                subject_visit=subject_visit,
                report_datetime=subject_visit.report_datetime,
                **options,
            )
        return obj



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from ..admin_site import intecomm_subject_admin
from ..forms import ClinicalNoteForm
from ..models import ClinicalNote
from .modeladmin_mixins import CrfModelAdmin


@admin.register(ClinicalNote, site=intecomm_subject_admin)
class ClinicalNoteAdmin(CrfModelAdmin):
    form = ClinicalNoteForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        ("Notes", {"fields": ("has_comment",)}),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {"crf_status": admin.VERTICAL}



# Generated by Django 4.1.2 on 2023-02-28 00:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0014_alter_historicalotherbaselinedata_activity_combined_mn_avg_day_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsins",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsins",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="results_abnormal",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Abnormal results at present at baseline or continuing from baseline not included.",
                max_length=25,
                verbose_name="Are any of the above results abnormal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="results_reportable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="If YES, this value will open Adverse Event Form. Grade 3 and 4 results present at baseline or continuing from baseline not included",
                max_length=25,
                verbose_name="If any results are abnormal, are results within grade 3 or above?",
            ),
        ),
    ]



from django import forms
from edc_crf.modelform_mixins import CrfSingletonModelFormMixin
from edc_he.form_validators import HealthEconomicsPatientFormValidator

from ...models import HealthEconomicsPatient
from ..mixins import CrfModelFormMixin
from .modelform_mixins import HealthEconomicsModelFormMixin


class HealthEconomicsPatientForm(
    CrfSingletonModelFormMixin,
    HealthEconomicsModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HealthEconomicsPatientFormValidator

    class Meta:
        model = HealthEconomicsPatient
        fields = "__all__"



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_sites.modelform_mixins import SiteModelFormMixin

from ..models import CommunityCareLocation


class CommunityCareLocationForm(
    SiteModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    class Meta:
        model = CommunityCareLocation
        fields = "__all__"
